﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Linq;
using HeuristicLab.Tracing;

namespace HeuristicLab.Hive.Server.LINQDataAccess {
  public abstract class BaseDao<TBusiness, TDatabaseEntity> {
    public static HiveDataContext Context {
      get {
        return ContextFactory.Context;
      }
    }

    protected void CommitChanges() {
      //TODO: DoWhile
      try {
        Context.SubmitChanges(ConflictMode.ContinueOnConflict);
      } catch (ChangeConflictException e) {
        
        Logger.Warn("Concurrency Exception! " + e.Message);
        foreach (ObjectChangeConflict conflict in Context.ChangeConflicts) { 
          Logger.Info("Conflicted: ");
          foreach (MemberChangeConflict memberChangeConflict in conflict.MemberConflicts) {
            Logger.Info("    Member in Conflict: " + memberChangeConflict.Member.Name);
            Logger.Info("    Database Value: " + memberChangeConflict.DatabaseValue);
            Logger.Info("    Original value: " + memberChangeConflict.OriginalValue);
            Logger.Info("    Current value: " + memberChangeConflict.CurrentValue);              
          }
          conflict.Resolve(RefreshMode.KeepChanges);
        }
        //next round!
        CommitChanges();
        //Context.SubmitChanges(ConflictMode.FailOnFirstConflict);
      }
    }

    public abstract TDatabaseEntity DtoToEntity(TBusiness source, TDatabaseEntity target);
    public abstract TBusiness EntityToDto(TDatabaseEntity source, TBusiness target);

  }
}
